function [R_final,eigen_vecs,eigen_vals] = A_func_noise_floor(R_init,rank,sigma_sq)

% [R_final,eigen_vecs,eigen_vals]=A_func_noise_floor(R_init,rank,sigma_sq)
% Function to enforce the covariance structural constraint R = Rc +
% sigma^2*I, where rank(Rc) <= rank < rank(R) and sigma^2 is known. 
%
% Inputs:
%-----------------------------------------------------------
% R_init   -  the covariance matrix estimate to be modified
% rank     -  the rank of Rc in the model R = Rc + sigma^2*I
% sigma_sq -  the thermal noise level
%
% Outputs:
%-----------------------------------------------------------
% R_final  -  the updated covariance estimate
%
% A. Shikhaliev
% Disclaimer: 
% Code provided as is with no warranty of fitness for any purpose.

% Perform the eigen-decomposition (equiv. to svd for psd matrices)
[eigen_vecs,eigen_vals] = svd(R_init);
eigen_vals = diag(eigen_vals);

% Replace the noise floor
optimal_c = sigma_sq;
eigen_vals((rank+1):end) = optimal_c;
eigen_vals(eigen_vals < optimal_c) = optimal_c;

% Construct covariance matrix with modified spectrum
R_final = eigen_vecs*diag(eigen_vals)*eigen_vecs';
end
